#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include "bmp_pixel.h"
#include "Nokia5110.h"
#include "cod.h"

#define NUM_OF_LEDS 9
/* After this threshold the leds will be lit for 1.5s */
#define MEDIUM_SCORE_THRESHOLD 20
/* After this threshold the leds will be lit for 1s */
#define HARD_SCORE_THRESHOLD 50
/* After this threshold the leds will be lit for 0.7s */
#define HELL_SCORE_THRESHOLD 100 // unused
/* After this threshold the leds will be lit for 0.5s */
#define ARE_YOU_MAD_THRESHOLD 500 // unused


/* Initial number of lives. When it gets to 0 you lose */
int lives = 5;
/* If the player hit a buton this "round" */
int hit_this_turn = 0;
/* Score */
int score = 0;
/* The led that is lit */
int led = -1;
/*Score from last game */
int last_score = 0;

/* Intrerupt handle for all pins of portA */
ISR(PCINT0_vect) {
	if ((PINA & (1 << PA0)) == 0) {
		if (led == 2 && hit_this_turn == 0) {
			score++;
			PORTB &= ~(1 << PB2);
		}
		else if (hit_this_turn == 0)
			lives--;
		hit_this_turn = 1;
	}
	else if ((PINA & (1 << PA1)) == 0) {
		if (led == 1 && hit_this_turn == 0) {
			score++;
			PORTB &= ~(1 << PB1);
		}
		else if (hit_this_turn == 0)
			lives--;
		hit_this_turn = 1;
	}
	else if ((PINA & (1 << PA2)) == 0) {
		if (led == 0 && hit_this_turn == 0) {
			score++;
			PORTB &= ~(1 << PB0);
		}
		else if (hit_this_turn == 0)
			lives--;
		hit_this_turn = 1;
	}
	else if ((PINA & (1 << PA3)) == 0) {
		if (led == 5 && hit_this_turn == 0) {
			score++;
			PORTB &= ~(1 << PB5);
		}
		else if (hit_this_turn == 0)
			lives--;
		hit_this_turn = 1;
	}
	else if ((PINA & (1 << PA4)) == 0) {
		if (led == 4 && hit_this_turn == 0) {
			score++;
			PORTB &= ~(1 << PB4);
		}
		else if (hit_this_turn == 0)
			lives--;
		hit_this_turn = 1;
	}
	else if ((PINA & (1 << PA5)) == 0) {
		if (led == 3 && hit_this_turn == 0) {
			score++;
			PORTB &= ~(1 << PB3);
		}
		else if (hit_this_turn == 0)
			lives--;
		hit_this_turn = 1;
	}
	else if ((PINA & (1 << PA6)) == 0) {
		if (led == 8 && hit_this_turn == 0) {
			score++;
			PORTC &= ~(1 << PC0);
		}
		else if (hit_this_turn == 0)
			lives--;
		hit_this_turn = 1;
	}
	else if ((PINA & (1 << PA7)) == 0) {
		if (led == 7 && hit_this_turn == 0) {
			score++;
			PORTB &= ~(1 << PB7);
		}
		else if (hit_this_turn == 0)
			lives--;
		hit_this_turn = 1;
	}
}

/* Intrerupt handle for PC1 */
ISR(PCINT2_vect) {
	if (led == 6 && hit_this_turn == 0) {
		score++;
		PORTB &= ~(1 << PB6);
	} else if (hit_this_turn == 0){
		lives--;
	}
	hit_this_turn = 1;
}

/* Intrerupt handle for Timer 1 Compare with OCR1A */
ISR(TIMER1_COMPA_vect)
{
	/* Save the old led and Turn on a random led */ 
	int old_led = led;
    led = rand() % NUM_OF_LEDS;

    /* Shut old led */
    switch(old_led) {
    	case 0:
    		PORTB &= ~(1 << PB0);
    		break;
    	case 1:
    		PORTB &= ~(1 << PB1);
    		break;
    	case 2:
    		PORTB &= ~(1 << PB2);
    		break;
    	case 3:
    		PORTB &= ~(1 << PB3);
    		break;
    	case 4:
    		PORTB &= ~(1 << PB4);
    		break;
    	case 5:
    		PORTB &= ~(1 << PB5);
    		break;
    	case 6:
    		PORTB &= ~(1 << PB6);
    		break;
    	case 7:
    		PORTB &= ~(1 << PB7);
    		break;
    	case 8:
    		PORTC &= ~(1 << PC0);
    		break;
    	default:
    		break;
    }

    /* Open new led */
    switch(led) {
    	case 0:
    		PORTB |= (1 << PB0);
    		break;
    	case 1:
    		PORTB |= (1 << PB1);
    		break;
    	case 2:
    		PORTB |= (1 << PB2);
    		break;
    	case 3:
    		PORTB |= (1 << PB3);
    		break;
    	case 4:
    		PORTB |= (1 << PB4);
    		break;
    	case 5:
    		PORTB |= (1 << PB5);
    		break;
    	case 6:
    		PORTB |= (1 << PB6);
    		break;
    	case 7:
    		PORTB |= (1 << PB7);
    		break;
    	case 8:
    		PORTC |= (1 << PC0);
    		break;
    	default:
    		break;
    }

    /* If no button was hit player looses a life */
    if(hit_this_turn == 0) {
		lives--;
    }

    /* reset "round" */
    hit_this_turn = 0;
}

void init()
{
	/* Set all pins of port A as input.
	 * 8 of the 9 buttuns are connected here. 
	 */
	DDRA &= ~(1 << PA0) & ~(1 << PA1) & ~(1 << PA2) & ~(1 << PA3);
	DDRA &= ~(1 << PA4) & ~(1 << PA5) & ~(1 << PA6) & ~(1 << PA7);

	/* Activate pull up for all buttons */
	PORTA |= (1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3) | (1 << PA4);
	PORTA |= (1 << PA5) | (1 << PA6) | (1 << PA7);

	/* Set PC1 as input. The last button is connected here */
	DDRC &= ~(1 << PC1);

	PORTC |= (1 << PC1);

	/* Set all pins of port B as output.
	 * 8 of the 9 LEDs are connected here.
	 */
	DDRB |= (1 << PB0) | (1 << PB1) | (1 << PB2) | (1 << PB3) | (1 << PB4);
	DDRB |= (1 << PB5) | (1 << PB6) | (1 << PB7);

	PORTB &= ~(1 << PB0) & ~(1 << PB1) & ~(1 << PB2) & ~(1 << PB3);
	PORTB &= ~(1 << PB4) & ~(1 << PB5) & ~(1 << PB6) & ~(1 << PB7);

	/* Set PC0 as output. The last LED is connected here */
	DDRC |= (1 << PC0);

	PORTC &= ~(1 << PC0);

	/* Activate intrerupts for PCINT7:0 (port A) and PCINT16 (from port C) */
	/* PCINT4 is disabled */
	PCICR |= (1 << PCIE0) | (1 << PCIE2);
	PCMSK0 |= (1 << PCINT0) | (1 << PCINT1) | (1 << PCINT2) | (1 << PCINT3);
	PCMSK0 |= (1 << PCINT5) | (1 << PCINT6) | (1 << PCINT7);
	PCMSK2 |= (1 << PCINT17);

	/* Setup the display */
	LCD_init();
	LCD_clear();	
}

void stop_leds()
{
	PORTB ^= PORTB;
	PORTC &= ~(1 << PC0);
}

/* set timer1 to give intrerupts */
void init_timer()
{
	/* Setting timer in CTC mode with top at OCR1A and a prescaler of 1024*/
	TCCR1B |= (1 << WGM12) | (1 << CS12) | (1 << CS10);
	/* Set OCR1A -- for 2s */
	OCR1A = 31250;
	TIMSK1 = (1 << OCIE1A);
}

void start_game()
{

	/* Init all variables */
	score = 0;
	lives = 5;

	/* Remove PCINT4 intrerupt */
	PCMSK0 &= ~(1 << PCINT4);

	/* Display start message */
	LCD_clear();
	LCD_write_string(0, 0, "Whac-A-Led");
	LCD_write_string(3, 3, "Press START");
	LCD_write_string(0, 5, "Score:");
	char chr_prev_score[score % 10 + 2];
	LCD_write_string(35, 5, itoa(last_score, chr_prev_score, 10));
	
	/* If START button is pressed go to next stepc*/
	while(1) {
		if ((PINA & (1 << PA4)) == 0) { 

			/* Display ready message */
			LCD_clear();
			LCD_write_string(0, 0, "Whac-A-Led");
			LCD_write_string(0, 3, "Starts in 1s");
			break;
		}
	}

	_delay_ms(1000);
	/* Activate intrerupt for center button */
	PCMSK0 |= (1 << PCINT4);

	/* Init timer */
	init_timer();

	while(1) {
		if (lives < 0)
			break;
		LCD_clear();
		LCD_write_string(0, 0, "Whac-A-Led");
		LCD_write_string(0, 3, "Score:");
		char chr_score[score % 10 + 2];
		itoa(score, chr_score, 10);
		LCD_write_string(35, 3, chr_score);
		LCD_write_string(0, 4, "Lives:");
		// char chr_lives[score % 10 + 2];
		// itoa(lives, chr_lives, 10);
		// LCD_write_string(30, 3, chr_lives);
		if(score > ARE_YOU_MAD_THRESHOLD) {
			OCR1A = 7813;
		} else if (score > HELL_SCORE_THRESHOLD) {
			OCR1A = 11004;
		} else if (score > HARD_SCORE_THRESHOLD) {
			OCR1A = 15625;
		} else if(score > MEDIUM_SCORE_THRESHOLD) {
			OCR1A = 23674;
		}
	}
	TIMSK1 &= ~(1 << OCIE1A);
	stop_leds();
	char chr_score[10];
	LCD_clear();
	LCD_write_string(0, 0, "Whac-A-Led");
	LCD_write_string(0, 3, "Your score:");
	LCD_write_string(0, 4, itoa(score, chr_score, 10));
	last_score = score;
	_delay_ms(2000);
}

int main(void) {
	
	/* Activate global intrerupts */
	sei();
	init();
	
	while (1) {
		start_game();
	}

	return 0;
}
